@@ -6,7 +6,7 @@
 *
 * Created   :   29.04.2011
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -184,6 +184,8 @@ class MAPIMapping {
             // "threadtopic"        => PR_CONVERSATION_TOPIC,
             "internetcpid"          => PR_INTERNET_CPID,
             "nativebodytype"        => PR_NATIVE_BODY_INFO,
+            "lastverbexecuted"      => PR_LAST_VERB_EXECUTED,
+            "lastverbexectime"      => PR_LAST_VERB_EXECUTION_TIME,
             );
     }
 
@@ -222,7 +224,7 @@ class MAPIMapping {
             "responserequested"     => PR_RESPONSE_REQUESTED,
             // timezone
             "alldayevent"           => "PT_BOOLEAN:PSETID_Appointment:0x8215",
-            "busystatus"            => "PT_LONG:PSETID_Appointment:0x8205",
+            "busystatus"            => "PT_LONG:PSETID_Appointment:0x8224",
             "rtf"                   => PR_RTF_COMPRESSED,
             "dtstamp"               => PR_LAST_MODIFICATION_TIME,
             "endtime"               => "PT_SYSTIME:PSETID_Appointment:0x820e",
@@ -395,6 +397,7 @@ class MAPIMapping {
             "completion"            => "PT_DOUBLE:PSETID_Task:0x8102",
             "status"                => "PT_LONG:PSETID_Task:0x8101",
             "icon"                  => PR_ICON_INDEX,
+            "owner"                 => "PT_STRING8:PSETID_Task:0x811F",
         );
     }
 
@@ -472,6 +475,9 @@ class MAPIMapping {
         return array(
             "body"                  => PR_BODY,
             "messageclass"          => PR_MESSAGE_CLASS,
+            "html"                  => PR_HTML,
+            "internetcpid"          => PR_INTERNET_CPID,
+
         );
     }
 
