@@ -21,7 +21,7 @@
 *
 * Created   :   26.12.2011
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -87,6 +87,16 @@ class StateManager {
         $this->synchedFolders = array();
     }
 
+    /**
+     * Prevents the StateMachine from removing old states
+     *
+     * @access public
+     * @return void
+     */
+    public function DoNotDeleteOldStates() {
+        $this->deleteOldStates = false;
+    }
+
     /**
      * Sets an ASDevice for the Statemanager to work with
      *
