@@ -14,7 +14,7 @@
 *
 * Created   :   06.01.2012
 *
-* Copyright 2007 - 2012 Zarafa Deutschland GmbH
+* Copyright 2007 - 2013 Zarafa Deutschland GmbH
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3,
@@ -53,6 +53,7 @@
 class SyncCollections implements Iterator {
     const ERROR_NO_COLLECTIONS = 1;
     const ERROR_WRONG_HIERARCHY = 2;
+    const OBSOLETE_CONNECTION = 3;
 
     private $stateManager;
 
@@ -107,6 +108,9 @@ class SyncCollections implements Iterator {
     public function LoadAllCollections($overwriteLoaded = false, $loadState = false, $checkPermissions = false) {
         $this->loadStateManager();
 
+        // this operation should not remove old state counters
+        $this->stateManager->DoNotDeleteOldStates();
+
         $invalidStates = false;
         foreach($this->stateManager->GetSynchedFolders() as $folderid) {
             if ($overwriteLoaded === false && isset($this->collections[$folderid]))
@@ -126,7 +130,7 @@ class SyncCollections implements Iterator {
     /**
      * Loads all collections known for the current device
      *
-     * @param boolean $folderid                 folder id to be loaded
+     * @param string $folderid                  folder id to be loaded
      * @param boolean $loadState                (opt) indicates if the collection sync state should be loaded, default true
      * @param boolean $checkPermissions         (opt) if set to true each folder will pass
      *                                          through a backend->Setup() to check permissions.
@@ -164,13 +168,13 @@ class SyncCollections implements Iterator {
             throw new StatusException(sprintf("SyncCollections->LoadCollection(): could not Setup() the backend for folder id '%s'", $spa->GetFolderId()), self::ERROR_WRONG_HIERARCHY);
 
         // add collection to object
-        $this->AddCollection($spa);
+        $addStatus = $this->AddCollection($spa);
 
         // load the latest known syncstate if requested
-        if ($loadState === true)
+        if ($addStatus && $loadState === true)
             $this->addparms[$folderid]["state"] = $this->stateManager->GetSyncState($spa->GetLatestSyncKey());
 
-        return true;
+        return $addStatus;
     }
 
     /**
@@ -182,7 +186,7 @@ class SyncCollections implements Iterator {
      * @return boolean
      */
     public function SaveCollection($spa) {
-        if (! $this->saveData)
+        if (! $this->saveData || !$spa->HasFolderId())
             return false;
 
         if ($spa->IsDataChanged()) {
@@ -211,8 +215,12 @@ class SyncCollections implements Iterator {
      * @return boolean
      */
     public function AddCollection($spa) {
+        if (! $spa->HasFolderId())
+            return false;
+
         $this->collections[$spa->GetFolderId()] = $spa;
 
+        ZLog::Write(LOGLEVEL_DEBUG, sprintf("SyncCollections->AddCollection(): Folder id '%s' : ref. PolicyKey '%s', ref. Lifetime '%s', last sync at '%s'", $spa->GetFolderId(), $spa->GetReferencePolicyKey(), $spa->GetReferenceLifetime(), $spa->GetLastSyncTime()));
         if ($spa->HasLastSyncTime() && $spa->GetLastSyncTime() > $this->lastSyncTime) {
             $this->lastSyncTime = $spa->GetLastSyncTime();
 
@@ -223,6 +231,8 @@ class SyncCollections implements Iterator {
             // use SyncParameters LifeTime as reference if available
             if ($spa->HasReferenceLifetime())
                 $this->refLifetime = $spa->GetReferenceLifetime();
+
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("SyncCollections->AddCollection(): Updated reference PolicyKey '%s', reference Lifetime '%s', Last sync at '%s'", $this->refPolicyKey, $this->refLifetime, $this->lastSyncTime));
         }
 
         return true;
@@ -285,6 +295,9 @@ class SyncCollections implements Iterator {
      * @return mixed            returns 'null' if nothing set
      */
     public function GetParameter($spa, $key) {
+        if (!$spa->HasFolderId())
+            return null;
+
         if (isset($this->addparms[$spa->GetFolderId()]) && isset($this->addparms[$spa->GetFolderId()][$key]))
             return $this->addparms[$spa->GetFolderId()][$key];
         else
@@ -367,28 +380,6 @@ class SyncCollections implements Iterator {
         return $this->lastSyncTime;
     }
 
-    /**
-     * Returns the timestamp of the last synchronization of a device.
-     *
-     * @param $device       an ASDevice
-     *
-     * @access public
-     * @return int                  timestamp
-     */
-    static public function GetLastSyncTimeOfDevice(&$device) {
-        // we need a StateManager for this operation
-        $stateManager = new StateManager();
-        $stateManager->SetDevice($device);
-
-        $sc = new SyncCollections();
-        $sc->SetStateManager($stateManager);
-
-        // load all collections of device without loading states or checking permissions
-        $sc->LoadAllCollections(true, false, false);
-
-        return $sc->GetLastSyncTime();
-    }
-
     /**
      * Checks if the currently known collections for changes for $lifetime seconds.
      * If the backend provides a ChangesSink the sink will be used.
@@ -432,7 +423,7 @@ class SyncCollections implements Iterator {
         $this->changes = array();
         $changesAvailable = false;
 
-        ZPush::GetTopCollector()->SetAsPushConnection();
+        ZPush::GetDeviceManager()->AnnounceProcessAsPush();
         ZPush::GetTopCollector()->AnnounceInformation(sprintf("lifetime %ds", $lifetime), true);
         ZLog::Write(LOGLEVEL_INFO, sprintf("SyncCollections->CheckForChanges(): Waiting for %s changes... (lifetime %d seconds)", (empty($classes))?'policy':'store', $lifetime));
 
@@ -458,6 +449,13 @@ class SyncCollections implements Iterator {
         // wait for changes
         $started = time();
         $endat = time() + $lifetime;
+
+        // always use policy key from the request if it was sent
+        $policyKey = $this->GetReferencePolicyKey();
+        if (Request::WasPolicyKeySent() && Request::GetPolicyKey() != 0) {
+            ZLog::Write(LOGLEVEL_DEBUG, sprintf("refpolkey:'%s', sent polkey:'%s'", $policyKey, Request::GetPolicyKey()));
+            $policyKey = Request::GetPolicyKey();
+        }
         while(($now = time()) < $endat) {
             // how long are we waiting for changes
             $this->waitingTime = $now-$started;
@@ -469,7 +467,7 @@ class SyncCollections implements Iterator {
 
             // Check if provisioning is necessary
             // if a PolicyKey was sent use it. If not, compare with the ReferencePolicyKey
-            if (PROVISIONING === true && $this->GetReferencePolicyKey() !== false && ZPush::GetDeviceManager()->ProvisioningRequired($this->GetReferencePolicyKey(), true))
+            if (PROVISIONING === true && $policyKey !== false && ZPush::GetDeviceManager()->ProvisioningRequired($policyKey, true))
                 // the hierarchysync forces provisioning
                 throw new StatusException("SyncCollections->CheckForChanges(): PolicyKey changed. Provisioning required.", self::ERROR_WRONG_HIERARCHY);
 
@@ -485,9 +483,8 @@ class SyncCollections implements Iterator {
 
                 // more than 60 secs to go?
                 if (($now + 60) < $endat) {
-                    ZLog::Write(LOGLEVEL_DEBUG, sprintf("SyncCollections->CheckForChanges(): Timeout forced after %ss from %ss due to other process", ($now-$started), $lifetime));
                     ZPush::GetTopCollector()->AnnounceInformation(sprintf("Forced timeout after %ds", ($now-$started)), true);
-                    return false;
+                    throw new StatusException(sprintf("SyncCollections->CheckForChanges(): Timeout forced after %ss from %ss due to other process", ($now-$started), $lifetime), self::OBSOLETE_CONNECTION);
                 }
             }
 
@@ -625,6 +622,23 @@ class SyncCollections implements Iterator {
         return $this->changes;
     }
 
+    /**
+     * Indicates if there are folders which are pingable
+     *
+     * @access public
+     * @return boolean
+     */
+    public function PingableFolders() {
+        $pingable = false;
+
+        foreach ($this->collections as $folderid => $spa) {
+            if ($spa->GetPingableFlag() == true)
+                $pingable = true;
+        }
+
+        return $pingable;
+    }
+
     /**
      * Indicates if the process did wait in a sink, polling or before running a
      * regular export to find changes
